unit conmenus;
{
	GearHead: Arena, a roguelike mecha CRPG
	Copyright (C) 2005 Joseph Hewitt

	This library is free software; you can redistribute it and/or modify it
	under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation; either version 2.1 of the License, or (at
	your option) any later version.

	The full text of the LGPL can be found in license.txt.

	This library is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
	General Public License for more details. 

	You should have received a copy of the GNU Lesser General Public License
	along with this library; if not, write to the Free Software Foundation,
	Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
}

	{ BUGS - If SelectMenu is handed an empty menu, all heck will }
	{  break loose. This can be a particular problem for SelectFile. }
	{  - If the last row is the maximum width of the menu, this }
	{  will screw up the display something awful. }

interface

{$IFDEF ASCII_SDL}
uses SDL;
{$ENDIF ASCII_SDL}

const
	{These two constants are used to tell the SELECT procedure whether or not}
	{the user is allowed to cancel.}
	RPMNormal = 0;
	RPMNoCancel = 1;

	RPMNoCleanup = 2; {If you want the menu left on the screen after we've finished, use this.}

type
	RPGMenuKeyPtr = ^RPGMenuKey;
	RPGMenuKey = Record
		k: Char;
		value: integer;		{The value returned when this key is pressed.}
		next: RPGMenuKeyPtr;
	end;

	RPGMenuItemPtr = ^RPGMenuItem;
	RPGMenuItem = Record
{$IFDEF PATCH_GH}
		indent: Integer;
{$ENDIF PATCH_GH}
		msg: string;		{The text which appears in the menu}
{$IFDEF PATCH_GH}
		value: LongInt;		{A value, returned by SelectMenu. -1 is reserved for Cancel}
{$ELSE PATCH_GH}
		value: integer;		{A value, returned by SelectMenu. -1 is reserved for Cancel}
{$ENDIF PATCH_GH}
		desc: string;		{Pointer to the item description. If Nil, no desc.}
		next: RPGMenuItemPtr;
	end;
	RPGMenu = Record
		active: boolean;
{$IFDEF ASCII_SDL}
		itemcolor,selcolor: TSDL_Color;
		x1,y1,x2,y2: byte;
		dtexcolor: TSDL_Color;
		dx1,dy1,dx2,dy2: byte; {fields relating to the optional description box.}
{$ELSE ASCII_SDL}
		itemcolor,selcolor,x1,y1,x2,y2: byte;
		dtexcolor,dx1,dy1,dx2,dy2: byte; {fields relating to the optional description box.}
{$ENDIF ASCII_SDL}
		mode: Byte;
		topitem,selectitem,numitem: integer; {fields holding info about the status of the menu.}
		FirstItem: RPGMenuItemPtr;
		FirstKey: RPGMenuKeyPtr;
	end;
	RPGMenuPtr = ^RPGMenu;

{$IFDEF PATCH_GH}
const
	Temporary_SelectItem: RPGMenuItemPtr = NIL;
{$ENDIF PATCH_GH}



{$IFDEF PATCH_GH}
Function AddRPGMenuItem(var RPM: RPGMenuPtr; indent: Integer; const msg: string; value: LongInt; const desc: string): RPGMenuItemPtr;
Function AddRPGMenuItem(var RPM: RPGMenuPtr; const msg: string; value: LongInt; const desc: string): RPGMenuItemPtr;
Function AddRPGMenuItem(var RPM: RPGMenuPtr; const msg: string; value: LongInt): RPGMenuItemPtr;
Function AddRPGMenuItem_Top(var RPM: RPGMenuPtr; indent: Integer; const msg: string; value: LongInt; const desc: string): RPGMenuItemPtr;
{$ELSE PATCH_GH}
Function AddRPGMenuItem(var RPM: RPGMenuPtr; const msg: string; value: integer; desc: string): RPGMenuItemPtr;
Function AddRPGMenuItem(var RPM: RPGMenuPtr; const msg: string; value: integer): RPGMenuItemPtr;
{$ENDIF PATCH_GH}
{$IFDEF PATCH_CHEAT}
Function PushRPGMenuItemFront( var RPM: RPGMenuPtr; indent: Integer; msg: string; value: LongInt; desc: string ): RPGMenuItemPtr;
{$ENDIF PATCH_CHEAT}
Procedure DisposeRPGMenuItem( var LList: RPGMenuItemPtr );
Procedure ClearMenu( RPM: RPGMenuPtr );
Procedure RemoveRPGMenuItem(RPM: RPGMenuPtr; var LMember: RPGMenuItemPtr);

Procedure AddRPGMenuKey(RPM: RPGMenuPtr; k: Char; value: Integer);
{$IFDEF ASCII_SDL}
Function CreateRPGMenu(icolor,scolor: TSDL_Color; x1,y1,x2,y2: byte): RPGMenuPtr;
Function CreateRPGMenu(icolor,scolor: TSDL_Color; Z: Integer): RPGMenuPtr;
{$ELSE ASCII_SDL}
Function CreateRPGMenu(icolor,scolor,x1,y1,x2,y2: byte): RPGMenuPtr;
Function CreateRPGMenu(icolor,scolor: byte; Z: Integer): RPGMenuPtr;
{$ENDIF ASCII_SDL}
Procedure AttachMenuDesc( RPM: RPGMenuPtr; Z: Integer );

{$IFDEF PATCH_GH}
Procedure DisposeRPGMenu(var RPM_arg: RPGMenuPtr);
{$ELSE PATCH_GH}
Procedure DisposeRPGMenu(var RPM: RPGMenuPtr);
{$ENDIF PATCH_GH}
Procedure DisplayMenu(RPM: RPGMenuPtr);
Function RPMLocateByPosition(RPM: RPGMenuPtr; i: integer): RPGMenuItemPtr;
{$IFDEF PATCH_GH}
Function SelectMenu( RPM: RPGMenuPtr ): LongInt;
{$ELSE PATCH_GH}
Function SelectMenu( RPM: RPGMenuPtr ): integer;
{$ENDIF PATCH_GH}
Procedure RPMSortAlpha(RPM: RPGMenuPtr);
{$IFDEF PATCH_CHEAT}
Procedure RPMSortAlpha_withSubItem(RPM: RPGMenuPtr);
{$ENDIF PATCH_CHEAT}

{$IFDEF PATCH_GH}
Function SetItemByValue( RPM: RPGMenuPtr ; V: LongInt ): RPGMenuItemPtr;
{$ELSE PATCH_GH}
Function SetItemByValue( RPM: RPGMenuPtr ; V: Integer ): RPGMenuItemPtr;
{$ENDIF PATCH_GH}
Procedure SetItemByPosition( RPM: RPGMenuPtr ; N: Integer );

{$IFDEF PATCH_GH}
Function BuildFileMenu( RPM: RPGMenuPtr; const SearchPattern: String; N: Integer ): Integer;
Function BuildFileMenu( RPM: RPGMenuPtr; const SearchPattern: String ): Integer;
{$ELSE PATCH_GH}
Procedure BuildFileMenu( RPM: RPGMenuPtr; const SearchPattern: String );
{$ENDIF PATCH_GH}
Function SelectFile( RPM: RPGMenuPtr ): String;

implementation

uses
{$IF DEFINED(ASCII_SDL)}
{$ELSEIF DEFINED(GUIMSWINMODE)}
{$ELSE}
	crt,
{$ENDIF}
	dos,
{$IFDEF PATCH_GH}
	errmsg,
  {$IFDEF PATCH_I18N}
	i18nmsg,
  {$ENDIF PATCH_I18N}
{$ELSE PATCH_GH}
  {$IFDEF DEBUG}
	errmsg,
  {$ENDIF DEBUG}
{$ENDIF PATCH_GH}
{$IF DEFINED(ASCII_SDL)}
	ascii_sdl,
{$ELSEIF DEFINED(GUIMSWINMODE)}
	w32crt,
{$ELSE}
{$ENDIF}
{$IFDEF PATCH_GH}
	pseudosmartpointer,
{$ENDIF PATCH_GH}
{$IFDEF DEBUG}
	gears,
{$ENDIF DEBUG}
{$IFDEF PATCH_GH}
	ui4gh,
{$ENDIF PATCH_GH}
{$IFDEF PATCH_I18N}
	texutil,
{$ENDIF PATCH_I18N}
{$IFDEF PATCH_CHEAT}
	menugear,
{$ENDIF PATCH_CHEAT}
	congfx,context;

{$IFDEF PATCH_GH}
var
	SearchPattern: String = '';
{$ENDIF PATCH_GH}

Function LastMenuItem(MIList: RPGMenuItemPtr): RPGMenuItemPtr;
	{This procedure will find the last item in the linked list.}
begin
	{While the menu item is pointing to a next menu item, it's not the last. duh.}
	{So, move through the list until we hit a Nil pointer.}
	while MIList^.next <> Nil do begin
		{Check the next one.}
		MIList := MIList^.next;
	end;
	{We've found a MI.next = Nil. Yay! Return its address.}
	LastMenuItem := MIList;
end;

{$IFDEF PATCH_GH}
Function AddRPGMenuItem(var RPM: RPGMenuPtr; indent: Integer; const msg: string; value: LongInt; const desc: string): RPGMenuItemPtr;
{$ELSE PATCH_GH}
Function AddRPGMenuItem(var RPM: RPGMenuPtr; const msg: string; value: integer; desc: string): RPGMenuItemPtr;
{$ENDIF PATCH_GH}
	{This procedure will add an item to the RPGToolMenu.}
	{The new item will be added as the last item in the list.}
var
	it: ^RPGMenuItem;		{Here's a pointer for the item we're creating.}
	temp: RPGMenuItemPtr;
begin
	{Allocate memory for it.}
	New(it);
{$IFDEF DEBUG}
	Trace_MemoryLeak('AddRPGMenuItem() New',it);
{$ENDIF DEBUG}

	{Check to make sure that the allocation succeeded.}
	if it = Nil then begin
		{Oops... something went wrong. Better let the user know.}
{$IFDEF PATCH_GH}
		ErrorMessage('Error: Popcorn Delta. AddRPGMenuItem messsed up.');
{$ELSE PATCH_GH}
		writeln('Error: Popcorn Delta. AddRPGMenuItem messsed up.');
{$ENDIF PATCH_GH}
{$IF DEFINED(ASCII_SDL)}
{$ELSEIF DEFINED(GUIMSWINMODE)}
		w32crt.RedrawScreen;
		w32crt.readkey;
{$ELSE}
		readkey;
{$ENDIF}
		exit;
	end;

	{Initialize it to the correct values.}
{$IFDEF PATCH_GH}
	it^.indent := indent;
{$ENDIF PATCH_GH}
	it^.msg := msg;
	it^.value := value;
	it^.next := Nil;
	it^.desc := desc; 	{The desc field is assigned the value of PChar since it}
				{is assumed that we arent responsible for the allocation,}
				{disposal, or contents of this string.}

	{Locate the last item in the list, then assign "it" to it.}
	{If the list is currently empty, stick "it" in as the first item.}
	if RPM^.firstitem = Nil then begin
		RPM^.firstitem := it;
	end else begin
		temp := LastMenuItem(RPM^.FirstItem);
		temp^.next := it;
	end;

	{Increment the NumItem field.}
	Inc(RPM^.numitem);

	AddRPGMenuItem := it;
end;

{$IFDEF PATCH_GH}
Function AddRPGMenuItem(var RPM: RPGMenuPtr; const msg: string; value: LongInt; const desc: string): RPGMenuItemPtr;
begin
	AddRPGMenuItem := AddRPGMenuItem( RPM , -1 , msg , value , desc );
end;
{$ENDIF PATCH_GH}

{$IFDEF PATCH_GH}
Function AddRPGMenuItem(var RPM: RPGMenuPtr; const msg: string; value: LongInt): RPGMenuItemPtr;
{$ELSE PATCH_GH}
Function AddRPGMenuItem(var RPM: RPGMenuPtr; const msg: string; value: integer): RPGMenuItemPtr;
{$ENDIF PATCH_GH}
	{ Just like the above, but no desc. }
begin
	AddRPGMenuItem := AddRPGMenuItem( RPM , msg , value , '' );
end;

{$IFDEF PATCH_GH}
Function AddRPGMenuItem_Top(var RPM: RPGMenuPtr; indent: Integer; const msg: string; value: LongInt; const desc: string): RPGMenuItemPtr;
	{This procedure will add an item to the RPGToolMenu.}
	{The new item will be added as the last item in the list.}
var
	it: ^RPGMenuItem;		{Here's a pointer for the item we're creating.}
	temp: RPGMenuItemPtr;
begin
	{Allocate memory for it.}
	New(it);
{$IFDEF DEBUG}
	Trace_MemoryLeak('AddRPGMenuItem_Top() New',it);
{$ENDIF DEBUG}

	{Check to make sure that the allocation succeeded.}
	if it = Nil then begin
		{Oops... something went wrong. Better let the user know.}
		ErrorMessage('Error: Popcorn Delta. AddRPGMenuItem_Top messsed up.');
{$IF DEFINED(ASCII_SDL)}
{$ELSEIF DEFINED(GUIMSWINMODE)}
		w32crt.RedrawScreen;
		w32crt.readkey;
{$ELSE}
		readkey;
{$ENDIF}
		exit;
	end;

	{Initialize it to the correct values.}
	it^.indent := indent;
	it^.msg := msg;
	it^.value := value;
	it^.next := Nil;
	it^.desc := desc; 	{The desc field is assigned the value of PChar since it}
				{is assumed that we arent responsible for the allocation,}
				{disposal, or contents of this string.}

	it^.next := RPM^.firstitem;
	RPM^.firstitem := it;

	{Increment the NumItem field.}
	Inc(RPM^.numitem);

	AddRPGMenuItem_Top := it;
end;
{$ENDIF PATCH_GH}

{$IFDEF PATCH_CHEAT}
Function PushRPGMenuItemFront( var RPM: RPGMenuPtr; indent: Integer; msg: string; value: LongInt; desc: string ): RPGMenuItemPtr;
var
	it: ^RPGMenuItem;		{Here's a pointer for the item we're creating.}
	temp: RPGMenuItemPtr;
begin
	{Allocate memory for it.}
	New(it);
{$IFDEF DEBUG}
	Trace_MemoryLeak('PushRPGMenuItemFront() New',it);
{$ENDIF DEBUG}

	{Check to make sure that the allocation succeeded.}
	if it = Nil then begin
		{Oops... something went wrong. Better let the user know.}
  {$IFDEF PATCH_GH}
		ErrorMessage('Error: Popcorn Delta. AddRPGMenuItem messsed up.');
  {$ELSE PATCH_GH}
		writeln('Error: Popcorn Delta. AddRPGMenuItem messsed up.');
  {$ENDIF PATCH_GH}
  {$IF DEFINED(ASCII_SDL)}
  {$ELSEIF DEFINED(GUIMSWINMODE)}
		w32crt.RedrawScreen;
		w32crt.readkey;
  {$ELSE}
	{	readkey;	}
  {$ENDIF}
		exit;
	end;

	{Initialize it to the correct values.}
{$IFDEF PATCH_GH}
	it^.indent := indent;
{$ENDIF PATCH_GH}
	it^.msg := msg;
	it^.value := value;
	it^.next := Nil;
	it^.desc := desc; 	{The desc field is assigned the value of PChar since it}
				{is assumed that we arent responsible for the allocation,}
				{disposal, or contents of this string.}

	{Locate the last item in the list, then assign "it" to it.}
	{If the list is currently empty, stick "it" in as the first item.}
	temp := RPM^.firstitem;
	it^.next := temp;
	RPM^.firstitem := it;

	{Increment the NumItem field.}
	Inc(RPM^.numitem);

	PushRPGMenuItemFront := it;
end;
{$ENDIF PATCH_CHEAT}

Procedure DisposeRPGMenuItem( var LList: RPGMenuItemPtr );
	{ Get rid of this list of items. }
	{ WARNING - If you call this procedure for a menu, it will not }
	{ change the value of the NumItems field!!! This will cause }
	{ problems when trying to use the menu. Unless you know exactly }
	{ what you're doing, use the ClearMenu procedure instead. }
var
	NextItem: RPGMenuItemPtr;
begin
	while LList <> Nil do begin
		NextItem := LList^.Next;
{$IFDEF DEBUG}
		Trace_MemoryLeak('DisposeRPGMenuItem() Dispose',LList);
{$ENDIF DEBUG}
{$IFDEF PATCH_GH}
		CheckAndNIL_Pointer('DisposeRPGMenuItem() Dispose',LList,True);
{$ENDIF PATCH_GH}
{$IFDEF PATCH_GH_PARANOID_SAFER}
		LList^.msg[1]  := '@';
		LList^.value   := -32767;
		LList^.desc[1] := '@';
		LList^.Next    := RPGMenuItemPtr(-1);
{$ENDIF PATCH_GH_PARANOID_SAFER}
		Dispose( LList );
		LList := NextItem;
	end;
end;

Procedure ClearMenu( RPM: RPGMenuPtr );
	{ Deallocate all the items in this menu, and set the number of }
	{ items to 0. }
begin
	DisposeRPGMenuItem( RPM^.FirstItem );
	RPM^.NumItem := 0;
	RPM^.SelectItem := 1;
	RPM^.topitem := 1;
end;

Procedure RemoveRPGMenuItem(RPM: RPGMenuPtr; var LMember: RPGMenuItemPtr);
	{Locate and extract member LMember from list LList.}
	{Then, dispose of LMember.}
var
	a,b: RPGMenuItemPtr;
begin
	{Initialize A and B}
	B := RPM^.FirstItem;
	A := Nil;

	{Locate LMember in the list. A will thereafter be either Nil,}
	{if LMember if first in the list, or it will be equal to the}
	{element directly preceding LMember.}
	while (B <> LMember) and (B <> Nil) do begin
		A := B;
		B := B^.next;
	end;

	if B = Nil then begin
		{Major FUBAR. The member we were trying to remove can't}
		{be found in the list.}
{$IFDEF PATCH_GH}
		ErrorMessage('ERROR- RemoveLink asked to remove a link that doesnt exist.');
{$ELSE PATCH_GH}
		writeln('ERROR- RemoveLink asked to remove a link that doesnt exist.');
{$ENDIF PATCH_GH}
		end
	else if A = Nil then begin
		{There's no element before the one we want to remove,}
		{i.e. it's the first one in the list.}
		RPM^.FirstItem := B^.Next;
{$IFDEF DEBUG}
		Trace_MemoryLeak('RemoveRPGMenuItem() Dispose',B);
{$ENDIF DEBUG}
{$IFDEF PATCH_GH}
		CheckAndNIL_Pointer('RemoveRPGMenuItem() Dispose',B,True);
{$ENDIF PATCH_GH}
{$IFDEF PATCH_GH_PARANOID_SAFER}
		B^.msg[1]  := '@';
		B^.value   := -32767;
		B^.desc[1] := '@';
		B^.Next    := RPGMenuItemPtr(-1);
{$ENDIF PATCH_GH_PARANOID_SAFER}
		Dispose(B);
		end
	else begin
		{We found the attribute we want to delete and have another}
		{one standing before it in line. Go to work.}
		A^.next := B^.next;
{$IFDEF DEBUG}
		Trace_MemoryLeak('RemoveRPGMenuItem() Dispose',B);
{$ENDIF DEBUG}
{$IFDEF PATCH_GH}
		CheckAndNIL_Pointer('RemoveRPGMenuItem() Dispose',B,True);
{$ENDIF PATCH_GH}
{$IFDEF PATCH_GH_PARANOID_SAFER}
		B^.msg[1]  := '@';
		B^.value   := -32767;
		B^.desc[1] := '@';
		B^.Next    := RPGMenuItemPtr(-1);
{$ENDIF PATCH_GH_PARANOID_SAFER}
		Dispose(B);
	end;

	{ Reduce the number of items in the menu. }
	Dec(RPM^.NumItem);
end;

Procedure AddRPGMenuKey(RPM: RPGMenuPtr; k: Char; value: Integer);
	{Add a dynamically defined RPGMenuKey to the menu.}
var
	it: RPGMenuKeyPtr;
begin
	New(it);
{$IFDEF DEBUG}
	Trace_MemoryLeak('AddRPGMenuKey() New',it);
{$ENDIF DEBUG}
	if it = Nil then begin
{$IFDEF PATCH_GH}
		ErrorMessage('ERROR- AddRPGMenuKey failed on memory allocation. Buy some RAM.');
{$ELSE PATCH_GH}
		writeln('ERROR- AddRPGMenuKey failed on memory allocation. Buy some RAM.');
{$ENDIF PATCH_GH}
		exit;
	end;

	{Initialize the values.}
	it^.k := k;
	it^.value := value;
	it^.next := RPM^.FirstKey;
	RPM^.FirstKey := it;
end;

{$IFDEF ASCII_SDL}
Function CreateRPGMenu( icolor,scolor: TSDL_Color; x1,y1,x2,y2: byte ): RPGMenuPtr;
{$ELSE ASCII_SDL}
Function CreateRPGMenu( icolor,scolor,x1,y1,x2,y2: byte ): RPGMenuPtr;
{$ENDIF ASCII_SDL}
	{This function creates a new RPGMenu record, and returns the address.}
var
	it: ^RPGMenu;			{Here's a pointer for the menu we're making.}
begin
	{Allocate memory for it.}
	New(it);
{$IFDEF DEBUG}
	Trace_MemoryLeak('CreateRPGMenu() New',it);
{$ENDIF DEBUG}

	{Check to make sure that we've actually initialized something.}
	if it = Nil then begin
		{Oops... something went wrong. Better let the user know.}
{$IFDEF PATCH_GH}
		ErrorMessage('Error: Popaboner Overflow. CreateRPGMenu messsed up. I make no promises.');
{$ELSE PATCH_GH}
		writeln('Error: Popaboner Overflow. CreateRPGMenu messsed up. I make no promises.');
{$ENDIF PATCH_GH}
{$IF DEFINED(ASCII_SDL)}
{$ELSEIF DEFINED(GUIMSWINMODE)}
		w32crt.RedrawScreen;
		w32crt.readkey;
{$ELSE}
		readkey;
{$ENDIF}
		exit;
	end;

	{Initialize the elements of the record.}
	it^.itemcolor := icolor;
	it^.selcolor := scolor;
	it^.x1 := x1;
	it^.y1 := y1;
	it^.x2 := x2;
	it^.y2 := y2;
	it^.Mode := RPMNormal;
	it^.FirstItem := Nil;
	it^.FirstKey := Nil;
	it^.dx1 := 0;	{A X1 value of 0 means there is no desc window.}
	{Might as well initialize the desc colors while}
	it^.dtexcolor := icolor; {we're here.}
	it^.active := False;

	{TopItem refers to the highest item on the screen display.}
	it^.topitem := 1;

	{SelectItem refers to the item currently being pointed at by the selector.}
	it^.selectitem := 1;

	{NumItem refers to the total number of items currently in the linked list.}
	it^.numitem := 0;

	{Return the address.}
	CreateRPGMenu := it;
end;

{$IFDEF ASCII_SDL}
Function CreateRPGMenu(icolor,scolor: TSDL_Color; Z: Integer): RPGMenuPtr;
{$ELSE ASCII_SDL}
Function CreateRPGMenu(icolor,scolor: byte; Z: Integer): RPGMenuPtr;
{$ENDIF ASCII_SDL}
	{ Create a new menu, dimension it after screen zone Z. }
begin
	CreateRPGMenu := CreateRPGMenu( icolor , scolor , ScreenZone[ Z , 1 ] , ScreenZone[ Z , 2 ] , ScreenZone[ Z , 3 ] , ScreenZone[ Z , 4 ] );
end;

Procedure AttachMenuDesc( RPM: RPGMenuPtr; Z: Integer );
	{ Set the area for description items to zone Z. }
begin
	RPM^.dx1 := ScreenZone[ Z , 1 ];
	RPM^.dy1 := ScreenZone[ Z , 2 ];
	RPM^.dx2 := ScreenZone[ Z , 3 ];
	RPM^.dy2 := ScreenZone[ Z , 4 ];
end;

{$IFDEF PATCH_GH}
Procedure DisposeRPGMenu(var RPM_arg: RPGMenuPtr);
{$ELSE PATCH_GH}
Procedure DisposeRPGMenu(var RPM: RPGMenuPtr);
{$ENDIF PATCH_GH}
	{This procedure is called when you want to get rid of the menu. It will deallocate}
	{the memory for the RPGMenu record and also for all of the linked RPGMenuItems.}
var
{$IFDEF PATCH_GH}
	RPM: RPGMenuPtr;
{$ENDIF PATCH_GH}
	c,d: RPGMenuKeyPtr;
begin
{$IFDEF PATCH_GH}
	RPM := RPM_arg;
{$ENDIF PATCH_GH}
	{Check to make sure that we've got a valid pointer here.}
	if RPM = Nil then begin
{$IFDEF PATCH_GH}
		ErrorMessage('ERROR: Joe is a Doofus. DisposeRPGMenu has been passed a null pointer.');
{$ELSE PATCH_GH}
		writeln('ERROR: Joe is a Doofus. DisposeRPGMenu has been passed a null pointer.');
{$ENDIF PATCH_GH}
{$IF DEFINED(ASCII_SDL)}
{$ELSEIF DEFINED(GUIMSWINMODE)}
		w32crt.RedrawScreen;
		w32crt.readkey;
{$ELSE}
		readkey;
{$ENDIF}
		exit;
	end;
{$IFDEF PATCH_GH}
	RPM_arg := NIL;
{$ENDIF PATCH_GH}

	{Save the location of the first menu item...}
	DisposeRPGMenuItem( RPM^.FirstItem );
	c := RPM^.FirstKey;
	{... then get rid of the menu record.}
{$IFDEF DEBUG}
	Trace_MemoryLeak('DisposeRPGMenu() Dispose',RPM);
{$ENDIF DEBUG}
{$IFDEF PATCH_GH}
	CheckAndNIL_Pointer('DisposeRPGMenu() Dispose',RPM,True);
{$ENDIF PATCH_GH}
{$IFDEF PATCH_GH_PARANOID_SAFER}
  {$IFDEF ASCII_SDL}
	RPM^.ItemColor.r := 255;
	RPM^.ItemColor.g := 255;
	RPM^.ItemColor.b := 255;
	RPM^.SelColor.r  := 255;
	RPM^.SelColor.g  := 255;
	RPM^.SelColor.b  := 255;
	RPM^.DtexColor.r := 255;
	RPM^.DtexColor.g := 255;
	RPM^.DtexColor.b := 255;
  {$ELSE ASCII_SDL}
	RPM^.ItemColor  := 126;
	RPM^.SelColor   := 126;
	RPM^.DtexColor  := 126;
  {$ENDIF ASCII_SDL}
	RPM^.mode       := 126;
	RPM^.TopItem    := -32767;
	RPM^.SelectItem := -32767;
	RPM^.NumItem    := -32767;
	RPM^.FirstItem  := RPGMenuItemPtr(-1);
	RPM^.FirstKey   := RPGMenuKeyPtr(-1);
{$ENDIF PATCH_GH_PARANOID_SAFER}
	Dispose(RPM);
	RPM := Nil;

	{Keep processing the menu items until we hit a Nil nextitem.}
	while c <> Nil do begin
		d := c^.next;
{$IFDEF DEBUG}
		Trace_MemoryLeak('DisposeRPGMenu() Dispose',c);
{$ENDIF DEBUG}
{$IFDEF PATCH_GH}
		CheckAndNIL_Pointer('DisposeRPGMenu() Dispose',c,True);
{$ENDIF PATCH_GH}
{$IFDEF PATCH_GH_PARANOID_SAFER}
		c^.k     := #126;
		c^.value := -32767;
		c^.Next  := RPGMenuKeyPtr(-1);
{$ENDIF PATCH_GH_PARANOID_SAFER}
		Dispose(c);
		c := d;
	end;
end;

Function RPMLocateByPosition(RPM: RPGMenuPtr; i: integer): RPGMenuItemPtr;
	{Locate the i'th element of the item list, then return its address.}
var
	a: RPGMenuItemPtr;	{Our pointer}
	t: integer;		{Our counter}
begin
	{Error check, first off.}
	if i > RPM^.numitem then begin
{$IFDEF PATCH_GH}
		ErrorMessage('ERROR: RPMLocateByPosition asked to find a message that doesnt exist.');
{$ELSE PATCH_GH}
		writeln('ERROR: RPMLocateByPosition asked to find a message that doesnt exist.');
{$ENDIF PATCH_GH}
{$IF DEFINED(ASCII_SDL)}
{$ELSEIF DEFINED(GUIMSWINMODE)}
		w32crt.RedrawScreen;
		w32crt.readkey;
{$ELSE}
		readkey;
{$ENDIF}
		exit;
	end;

	a := RPM^.FirstItem;
	t := 1;

	if i > 1 then begin
		for t := 2 to i do
			a := a^.next;
	end;

	RPMLocateByPosition := a;
end;

{$IFDEF PATCH_GH}
Function MenuWidth( RPM: RPGMenuPtr ): Integer;
	{ Return the height of the menu, in text rows. }
var
	MW: Integer;
begin
	MW := RPM^.X2 - RPM^.X1 + 1;
	if Show_MenuScrollbar then begin
		Dec( MW );
	end;
	if MW < 1 then MW := 1;
	MenuWidth := MW;
end;
{$ENDIF PATCH_GH}

Function MenuHeight( RPM: RPGMenuPtr ): Integer;
	{ Return the height of the menu, in text rows. }
var
	MH: Integer;
begin
	MH := RPM^.Y2 - RPM^.Y1 + 1;
{$IFDEF PATCH_GH}
	if Show_MenuPage then begin
		Dec( MH );
	end;
{$ENDIF PATCH_GH}
	if MH < 1 then MH := 1;
	MenuHeight := MH;
end;

Procedure SetMenuClipZone( RPM: RPGMenuPtr );
	{ Set the clip area for this menu. }
begin
{$IF DEFINED(ASCII_SDL)}
	ascii_sdl.Window( RPM^.X1 , RPM^.Y1 , RPM^.X2 , RPM^.Y2 );
{$ELSEIF DEFINED(GUIMSWINMODE)}
	w32crt.Window(RPM^.X1,RPM^.Y1,RPM^.X2,RPM^.Y2);
{$ELSE}
	Window(RPM^.X1,RPM^.Y1,RPM^.X2,RPM^.Y2);
{$ENDIF}
end;

Procedure RPMRefreshDesc(RPM: RPGMenuPtr);
	{Refresh the menu description box, if appropriate.}
begin
	{Check to make sure that this menu has a description box, first off.}
	if RPM^.dx1 > 0 then begin
		GameMSG(RPMLocateByPosition(RPM,RPM^.selectitem)^.desc,RPM^.DX1,RPM^.DY1,RPM^.DX2,RPM^.DY2,RPM^.dtexcolor);
	end;
end;

Procedure DisplayMenu(RPM: RPGMenuPtr);
	{Display the menu on the screen.}
{$IFDEF PATCH_GH}
const
	MoreMark: String = '*';
	CurrentPageMark: String = '-';
{$ENDIF PATCH_GH}
var
	topitem: RPGMenuItemPtr;
	a: RPGMenuItemPtr;		{A pointer to be used while printing.}
	t: integer;
	width,height: integer;		{The width of the menu display.}
{$IFDEF PATCH_I18N}
	maxwidth, trimedlength: integer;
{$ENDIF PATCH_I18N}
{$IFDEF PATCH_GH}
	MenuItemStr: String;
	LastItem: Integer;
{$ENDIF PATCH_GH}
{$IFDEF PATCH_GH}
	Y: Integer;
{$ENDIF PATCH_GH}
begin
	{Error check- make sure the menu has items in it.}
	if RPM^.FirstItem = Nil then Exit;
{$IFDEF PATCH_GH}
	if RPM^.NumItem < RPM^.SelectItem then begin
		RPM^.SelectItem := RPM^.NumItem;
	end else if RPM^.SelectItem < 1 then begin
		RPM^.SelectItem := 1;
	end;
	if RPM^.NumItem < RPM^.TopItem then begin
		RPM^.TopItem := RPM^.NumItem;
	end else if RPM^.TopItem < 1 then begin
		RPM^.TopItem := 1;
	end;
{$ENDIF PATCH_GH}

{$IFDEF PATCH_GH}
	Temporary_SelectItem := RPMLocateByPosition(RPM,RPM^.selectitem);
{$ENDIF PATCH_GH}
{$IFDEF PATCH_GH}
	Temporary_SelectItem := NIL;
{$ENDIF PATCH_GH}

	{Display each menu item.}

	{Open an appropriately sized window and clear that area.}
	SetMenuClipZone( RPM );
{$IF DEFINED(ASCII_SDL)}
	ascii_sdl.ClrScr;
{$ELSEIF DEFINED(GUIMSWINMODE)}
	w32crt.ClrScr;
{$ELSE}
	ClrScr;
{$ENDIF}

	{Calculate the width and the height of the menu.}
{$IFDEF PATCH_GH}
	width := MenuWidth( RPM );
{$ELSE PATCH_GH}
	width := RPM^.X2 - RPM^.X1 + 1;
{$ENDIF PATCH_GH}
	height := MenuHeight( rpm );

	{Locate the top of the menu.}
	topitem := RPMLocateByPosition(RPM,RPM^.topitem);

	a := topitem;
	for t := 1 to Height do begin
		{If we're at the currently selected item, highlight it.}
{$IF DEFINED(ASCII_SDL)}
		if ((t + RPM^.topitem - 1) = RPM^.selectitem) and RPM^.Active then
			ascii_sdl.TextColor(RPM^.selcolor)
		else
			ascii_sdl.TextColor(RPM^.itemcolor);

		ascii_sdl.GotoXY(1,t);
{$ELSEIF DEFINED(GUIMSWINMODE)}
		if ((t + RPM^.topitem - 1) = RPM^.selectitem) and RPM^.Active then
			w32crt.TextColor(RPM^.selcolor)
		else
			w32crt.TextColor(RPM^.itemcolor);

		w32crt.GotoXY(1,t);
{$ELSE}
		if ((t + RPM^.topitem - 1) = RPM^.selectitem) and RPM^.Active then
			TextColor(RPM^.selcolor)
		else
			TextColor(RPM^.itemcolor);

		GotoXY(1,t);
{$ENDIF}

{$IFDEF PATCH_I18N}
		maxwidth := width;
		if T = Height then maxwidth := (width - 1);
		trimedlength := MBCharTrimedLength( a^.msg, maxwidth );
		if (0 < trimedlength) then begin
			WriteMBCharStr(Copy(a^.msg,1,trimedlength), maxwidth );
		end;
{$ELSE PATCH_I18N}
		if T = Height then begin
			write(Copy(a^.msg,1,width - 1));
		end else begin
			write(Copy(a^.msg,1,width));
		end;
{$ENDIF PATCH_I18N}

{$IFDEF PATCH_GH}
		if Show_MenuScrollbar then begin
			if (1 = t) and (1 < RPM^.topitem) then begin
  {$IF DEFINED(ASCII_SDL)}
				ascii_sdl.TextColor(RPM^.selcolor);
				ascii_sdl.GotoXY(width,t);
				ascii_sdl.Write(MoreMark);
  {$ELSEIF DEFINED(GUIMSWINMODE)}
				w32crt.TextColor(RPM^.selcolor);
				w32crt.GotoXY(width+1,t);
				w32crt.Write(MoreMark);
  {$ELSE}
				TextColor(RPM^.selcolor);
				GotoXY(width,t);
				Write(MoreMark);
  {$ENDIF}
			end;
			if (Height = t) and ((RPM^.topitem + Height - 1) < RPM^.numitem) then begin
  {$IF DEFINED(ASCII_SDL)}
				ascii_sdl.TextColor(RPM^.selcolor);
				ascii_sdl.GotoXY(width,t);
				ascii_sdl.Write(MoreMark);
  {$ELSEIF DEFINED(GUIMSWINMODE)}
				w32crt.TextColor(RPM^.selcolor);
				w32crt.GotoXY(width+1,t);
				w32crt.Write(MoreMark);
  {$ELSE}
				TextColor(RPM^.selcolor);
				GotoXY(width,t);
				Write(MoreMark);
  {$ENDIF}
			end;
		end;
{$ENDIF PATCH_GH}

		a := a^.next;

		{Check to see if we've prematurely encountered the end of the list.}
		if a = Nil then
			break;
	end;

{$IFDEF PATCH_GH}
	if Show_MenuScrollbar and (Height < RPM^.numitem) then begin
		Y := (((RPM^.topitem + Height div 2 - 1) * (Height - 2)) div RPM^.numitem) + 1 + 1;
		if ( RPM^.topitem <= 1 ) then begin
			Y := 2;
		end else if ( RPM^.numitem <= (RPM^.topitem + Height - 1) ) then begin
			Y := Height - 1;
		end;
  {$IF DEFINED(ASCII_SDL)}
		ascii_sdl.TextColor(RPM^.selcolor);
		ascii_sdl.GotoXY(width,Y);
		ascii_sdl.Write(CurrentPageMark);
  {$ELSEIF DEFINED(GUIMSWINMODE)}
		w32crt.TextColor(RPM^.selcolor);
		w32crt.GotoXY(width+1,Y);
		w32crt.Write(CurrentPageMark);
  {$ELSE}
		TextColor(RPM^.selcolor);
		GotoXY(width,Y);
		Write(CurrentPageMark);
  {$ENDIF}
	end;
	if Show_MenuPage then begin
		LastItem := RPM^.topitem + Height - 1;
		if RPM^.numitem < LastItem then begin
			LastItem := RPM^.numitem;
		end;
		MenuItemStr := '(' + BStr(RPM^.topitem) + '-' + BStr(LastItem) + '/' + BStr(RPM^.numitem) + ')';
  {$IF DEFINED(ASCII_SDL)}
		ascii_sdl.TextColor(RPM^.itemcolor);
		ascii_sdl.GotoXY(1,Height+1);
  {$ELSEIF DEFINED(GUIMSWINMODE)}
		w32crt.TextColor(RPM^.itemcolor);
		w32crt.GotoXY(1,Height+1);
  {$ELSE}
		TextColor(RPM^.itemcolor);
		GotoXY(1,Height+1);
  {$ENDIF}
  {$IFDEF PATCH_I18N}
		maxwidth := width - 1;
		trimedlength := MBCharTrimedLength( MenuItemStr, maxwidth );
		if (0 < trimedlength) then begin
			WriteMBCharStr(Copy(MenuItemStr,1,trimedlength), maxwidth );
		end;
  {$ELSE PATCH_I18N}
		write(Copy(MenuItemStr,1,width - 1));
  {$ENDIF PATCH_I18N}
	end;
{$ENDIF PATCH_GH}

	{Restore the window to its regular size.}
	MaxCLipZone;

	{If there's an associated Desc field, display it now.}
	RPMRefreshDesc(RPM);
end;

Procedure RPMReposition(RPM: RPGMenuPtr);
	{The selected item has just changed, and is no longer visible on screen.}
	{Adjust the RPGMenu's topitem field to an appropriate value.}
begin
	{When this function is called, there are two possibilities: either the}
	{selector has moved off the bottom of the page or the top.}

	if RPM^.selectitem < RPM^.topitem then begin
		{The selector has moved off the bottom of the list. The new page}
		{display should start with SelectItem on the bottom.}
		RPM^.topitem := RPM^.selectitem - MenuHeight( RPM ) + 1;

		{Error check- if this moves topitem below 1, that's bad.}
		if RPM^.topitem < 1 then
			RPM^.topitem := 1;
		end
	else begin
		{The selector has moved off the top of the list. The new page should}
		{start with SelectItem at the top, unless this would make things look}
		{funny.}
		if ((RPM^.selectitem + MenuHeight( RPM ) - 1) > RPM^.numitem) then begin
			{There will be whitespace at the bottom of the menu if we assign}
			{SelectItem to TopItem. Make TopItem equal to the effective last}
			{page.}
			RPM^.topitem := RPM^.numitem - MenuHeight( RPM ) + 1;
			if RPM^.topitem < 1 then RPM^.topitem := 1;
			end
		else
			RPM^.topitem := RPM^.selectitem;
	end;

end;

{$IFDEF PATCH_GH}
Procedure RPMReposition_center( RPM: RPGMenuPtr );
begin
	{ Check a limit. }
	if (RPM^.selectitem < 1) then begin
		RPM^.selectitem := 1
	end else if (RPM^.numitem < RPM^.selectitem) then begin
		RPM^.selectitem := RPM^.numitem
	end;

	{ Auto reposition mode. }
	if (RPM^.topitem < 1) then begin
		RPM^.topitem := RPM^.selectitem - ( MenuHeight(RPM) div 2 );
		if (RPM^.topitem < 1) then begin
			RPM^.topitem := 1;
		end;
	end else if (RPM^.numitem < RPM^.topitem) then begin
		RPM^.topitem := RPM^.selectitem - MenuHeight(RPM) + 1;
		if (RPM^.topitem < 1) then begin
			RPM^.topitem := 1;
		end;
	end else if (RPM^.selectitem < RPM^.topitem) then begin
		RPM^.topitem := RPM^.selectitem - MenuHeight(RPM) + 1;
		if RPM^.topitem < 1 then begin
			RPM^.topitem := 1;
		end;
	end else if (RPM^.topitem + MenuHeight(RPM) <= RPM^.selectitem) then begin
		RPM^.topitem := RPM^.selectitem;
		if RPM^.numitem < RPM^.topitem then begin
			RPM^.topitem := RPM^.numitem;
		end;
	end;

end;
{$ENDIF PATCH_GH}

Procedure RPMUpKey(RPM: RPGMenuPtr);
	{Someone just pressed the UP key, and we're gonna process that input.}
	{PRECONDITIONS: RPM has been initialized properly, and is currently being}
	{  displayed on the screen.}
var
	width: integer;		{The width of the menu window}
{$IFDEF PATCH_I18N}
	maxwidth, trimedlength: integer;
{$ENDIF PATCH_I18N}
begin
	{Lets set up the window.}
	SetMenuClipZone( RPM );

	{Calculate the width of the menu.}
{$IFDEF PATCH_GH}
	width := MenuWidth( RPM );
{$ELSE PATCH_GH}
	width := RPM^.X2 - RPM^.X1 + 1;
{$ENDIF PATCH_GH}

	{De-indicate the old selected item.}
{$IF DEFINED(ASCII_SDL)}
	{Change color to the regular item color...}
	ascii_sdl.TextColor(RPM^.itemcolor);
	{Then reprint the text of the previously selected item.}
	ascii_sdl.GotoXY(1,RPM^.selectitem - RPM^.topitem + 1);
{$ELSEIF DEFINED(GUIMSWINMODE)}
	{Change color to the regular item color...}
	w32crt.TextColor(RPM^.itemcolor);
	{Then reprint the text of the previously selected item.}
	w32crt.GotoXY(1,RPM^.selectitem - RPM^.topitem + 1);
{$ELSE}
	{Change color to the regular item color...}
	TextColor(RPM^.itemcolor);
	{Then reprint the text of the previously selected item.}
	GotoXY(1,RPM^.selectitem - RPM^.topitem + 1);
{$ENDIF}
{$IFDEF PATCH_I18N}
	maxwidth := width;
	if ( RPM^.selectitem - RPM^.topitem + 1 ) = MenuHeight( RPM ) then maxwidth := (width - 1);
	trimedlength := MBCharTrimedLength( RPMLocateByPosition(RPM,RPM^.selectitem)^.msg, maxwidth );
	if (0 < trimedlength) then begin
		WriteMBCharStr(Copy(RPMLocateByPosition(RPM,RPM^.selectitem)^.msg,1,trimedlength), maxwidth );
	end;
{$ELSE PATCH_I18N}
	if ( RPM^.selectitem - RPM^.topitem + 1 ) = MenuHeight( RPM ) then begin
		write(Copy(RPMLocateByPosition(RPM,RPM^.selectitem)^.msg,1,width - 1));
	end else begin
		write(Copy(RPMLocateByPosition(RPM,RPM^.selectitem)^.msg,1,width));
	end;
{$ENDIF PATCH_I18N}

	{Decrement the selected item by one.}
	Dec(RPM^.selectitem);
	{If this causes it to go beneath one, wrap around to the last item.}
	if RPM^.selectitem = 0 then
		RPM^.selectitem := RPM^.numitem;

	{If the movement takes the selected item off the screen, do a redisplay.}
	{Otherwise, indicate the newly selected item.}
	if (RPM^.selectitem < RPM^.topitem) or ((RPM^.selectitem - RPM^.topitem) >= MenuHeight( RPM )) then begin
		{Determine an appropriate new value for topitem.}
		RPMReposition(RPM);

		{Redisplay the menu.}
		DisplayMenu(RPM);

		end
	else begin
{$IF DEFINED(ASCII_SDL)}
		ascii_sdl.TextColor(RPM^.selcolor);
		ascii_sdl.GotoXY(1,RPM^.selectitem - RPM^.topitem + 1);
{$ELSEIF DEFINED(GUIMSWINMODE)}
		w32crt.TextColor(RPM^.selcolor);
		w32crt.GotoXY(1,RPM^.selectitem - RPM^.topitem + 1);
{$ELSE}
		TextColor(RPM^.selcolor);
		GotoXY(1,RPM^.selectitem - RPM^.topitem + 1);
{$ENDIF}
{$IFDEF PATCH_I18N}
		maxwidth := width;
		if ( RPM^.selectitem - RPM^.topitem + 1 ) = MenuHeight( RPM ) then maxwidth := (width - 1);
		trimedlength := MBCharTrimedLength( RPMLocateByPosition(RPM,RPM^.selectitem)^.msg, maxwidth );
		if (0 < trimedlength) then begin
			WriteMBCharStr(Copy(RPMLocateByPosition(RPM,RPM^.selectitem)^.msg,1,trimedlength), maxwidth );
		end;
{$ELSE PATCH_I18N}
		if ( RPM^.selectitem - RPM^.topitem + 1 ) = MenuHeight( RPM ) then begin
			write(Copy(RPMLocateByPosition(RPM,RPM^.selectitem)^.msg,1,width-1));
		end else begin
			write(Copy(RPMLocateByPosition(RPM,RPM^.selectitem)^.msg,1,width));
		end;
{$ENDIF PATCH_I18N}

		MaxClipZone;

		{If this menu features item descriptions, better refresh the text.}
		if RPM^.dx1 > 0 then begin
			RPMRefreshDesc(RPM);
		end;
	end;

end;

Procedure RPMDownKey(RPM: RPGMenuPtr);
	{Someone just pressed the DOWN key, and we're gonna process that input.}
	{PRECONDITIONS: RPM has been initialized properly, and is currently being}
	{  displayed on the screen.}
var
	width: integer;		{The width of the menu window}
{$IFDEF PATCH_I18N}
	maxwidth, trimedlength: integer;
{$ENDIF PATCH_I18N}
begin
	{Lets set up the window.}
	SetMenuClipZone( RPM );

	{Calculate the width of the menu.}
{$IFDEF PATCH_GH}
	width := MenuWidth( RPM );
{$ELSE PATCH_GH}
	width := RPM^.X2 - RPM^.X1 + 1;
{$ENDIF PATCH_GH}

	{De-indicate the item.}
	{Change color to the normal text color, then reprint the item's message.}
{$IF DEFINED(ASCII_SDL)}
	ascii_sdl.TextColor(RPM^.itemcolor);
	ascii_sdl.GotoXY(1,RPM^.selectitem - RPM^.topitem + 1);
{$ELSEIF DEFINED(GUIMSWINMODE)}
	w32crt.TextColor(RPM^.itemcolor);
	w32crt.GotoXY(1,RPM^.selectitem - RPM^.topitem + 1);
{$ELSE}
	TextColor(RPM^.itemcolor);
	GotoXY(1,RPM^.selectitem - RPM^.topitem + 1);
{$ENDIF}
{$IFDEF PATCH_I18N}
	maxwidth := width;
	if ( RPM^.selectitem - RPM^.topitem + 1 ) = MenuHeight( RPM ) then maxwidth := (width - 1);
	trimedlength := MBCharTrimedLength( RPMLocateByPosition(RPM,RPM^.selectitem)^.msg, maxwidth );
	if (0 < trimedlength) then begin
		WriteMBCharStr(Copy(RPMLocateByPosition(RPM,RPM^.selectitem)^.msg,1,trimedlength), maxwidth );
	end;
{$ELSE PATCH_I18N}
	if ( RPM^.selectitem - RPM^.topitem + 1 ) = MenuHeight( RPM ) then begin
		write(Copy(RPMLocateByPosition(RPM,RPM^.selectitem)^.msg,1,width - 1));
	end else begin
		write(Copy(RPMLocateByPosition(RPM,RPM^.selectitem)^.msg,1,width));
	end;
{$ENDIF PATCH_I18N}

	{Increment the selected item.}
	Inc(RPM^.selectitem);
	{If this takes the selection out of bounds, restart at the first item.}
	if RPM^.selectitem = RPM^.numitem + 1 then
		RPM^.selectitem := 1;

	{If the movement takes the selected item off the screen, do a redisplay.}
	{Otherwise, indicate the newly selected item.}
	if (RPM^.selectitem < RPM^.topitem) or ((RPM^.selectitem - RPM^.topitem) >= MenuHeight( RPM ) ) then begin
		{Determine an appropriate new value for topitem.}
		RPMReposition(RPM);

		{Redisplay the menu.}
		DisplayMenu(RPM);

		end
	else begin
{$IF DEFINED(ASCII_SDL)}
		ascii_sdl.TextColor(RPM^.selcolor);
		ascii_sdl.GotoXY(1,RPM^.selectitem - RPM^.topitem + 1);
{$ELSEIF DEFINED(GUIMSWINMODE)}
		w32crt.TextColor(RPM^.selcolor);
		w32crt.GotoXY(1,RPM^.selectitem - RPM^.topitem + 1);
{$ELSE}
		TextColor(RPM^.selcolor);
		GotoXY(1,RPM^.selectitem - RPM^.topitem + 1);
{$ENDIF}
{$IFDEF PATCH_I18N}
		maxwidth := width;
		if ( RPM^.selectitem - RPM^.topitem + 1 ) = MenuHeight( RPM ) then maxwidth := (width - 1);
		trimedlength := MBCharTrimedLength( RPMLocateByPosition(RPM,RPM^.selectitem)^.msg, maxwidth );
		if (0 < trimedlength) then begin
			WriteMBCharStr(Copy(RPMLocateByPosition(RPM,RPM^.selectitem)^.msg,1,trimedlength), maxwidth );
		end;
{$ELSE PATCH_I18N}
		if ( RPM^.selectitem - RPM^.topitem + 1 ) = MenuHeight( RPM ) then begin
			write(Copy(RPMLocateByPosition(RPM,RPM^.selectitem)^.msg,1,width-1));
		end else begin
			write(Copy(RPMLocateByPosition(RPM,RPM^.selectitem)^.msg,1,width));
		end;
{$ENDIF PATCH_I18N}

		{Restore the window to its regular size.}
		MaxClipZone;

		{If this menu features item descriptions, better refresh the text.}
		if RPM^.dx1 > 0 then begin
			RPMRefreshDesc(RPM);
		end;
	end;

end;

{$IFDEF PATCH_GH}
Procedure RPMPgUpKey(RPM: RPGMenuPtr);
begin
	RPM^.selectitem := RPM^.selectitem - MenuHeight( RPM );
	if RPM^.selectitem < 1 then
		RPM^.selectitem := 1;

	while RPM^.selectitem < RPM^.topitem do begin
		RPM^.topitem := RPM^.topitem - MenuHeight( RPM );
		if RPM^.topitem < 1 then begin
			RPM^.topitem := 1;
			break;
		end;
	end;

	DisplayMenu(RPM);
end;

Procedure RPMPgDownKey(RPM: RPGMenuPtr);
begin
	RPM^.selectitem := RPM^.selectitem + MenuHeight( RPM );
	if RPM^.selectitem > RPM^.numitem then
		RPM^.selectitem := RPM^.numitem;

	while RPM^.selectitem >= (RPM^.topitem + MenuHeight(RPM)) do begin
		RPM^.topitem := RPM^.topitem + MenuHeight( RPM );
		if RPM^.topitem > RPM^.numitem then begin
			RPM^.topitem := RPM^.numitem;
			break;
		end;
	end;

	DisplayMenu(RPM);
end;

Procedure RPMScrollUpKey(RPM: RPGMenuPtr);
begin
	Dec(RPM^.selectitem);
	if RPM^.selectitem < 1 then
		RPM^.selectitem := 1;
	Dec(RPM^.topitem);
	if RPM^.topitem < 1 then
		RPM^.topitem := 1;

	DisplayMenu(RPM);
end;

Procedure RPMScrollDownKey(RPM: RPGMenuPtr);
begin
	Inc(RPM^.selectitem);
	if RPM^.numitem < RPM^.selectitem then
		RPM^.selectitem := RPM^.numitem;
	Inc(RPM^.topitem);
	if (RPM^.numitem > MenuHeight( RPM )) then begin
		if RPM^.topitem > (RPM^.numitem - MenuHeight( RPM ) + 1) then RPM^.topitem := RPM^.numitem - MenuHeight( RPM ) + 1;
	end else RPM^.topitem := 1;

	DisplayMenu(RPM);
end;
{$ENDIF PATCH_GH}

{$IFDEF PATCH_GH}
Procedure RPMSearchForwardKey( RPM: RPGMenuPtr );
var
	key: String;
	sel: Integer;
	RPM_Item: RPGMenuItemPtr;
begin
  {$IFDEF PATCH_I18N}
	key := GetStringFromUser( I18N_MsgString( 'RPMSearchForwardKey' , 'SearchForward' ) , SearchPattern );
  {$ELSE PATCH_I18N}
	key := GetStringFromUser( 'SearchForward' , SearchPattern );
  {$ENDIF PATCH_I18N}

	sel := RPM^.SelectItem;
	RPM_Item := RPMLocateByPosition( RPM , sel );
	repeat
		Inc( sel );
		RPM_Item := RPM_Item^.Next;
		if ( RPM^.NumItem < sel ) then begin
			sel := 1;
			RPM_Item := RPM^.FirstItem;
		end;
		if 0 < Pos( UpCase( Key ) , UpCase( RPM_Item^.msg ) ) then begin
			RPM^.SelectItem := sel;
			SearchPattern := key;
			break;
		end;
	until ( sel = RPM^.SelectItem );

	RPMReposition_center( RPM );
	DisplayMenu(RPM);
end;

Procedure RPMSearchBackwardKey( RPM: RPGMenuPtr );
var
	key: String;
	sel: Integer;
	RPM_Item: RPGMenuItemPtr;
begin
  {$IFDEF PATCH_I18N}
	key := GetStringFromUser( I18N_MsgString( 'RPMSearchBackwardKey' , 'SearchBackward' ) , SearchPattern );
  {$ELSE PATCH_I18N}
	key := GetStringFromUser( 'SearchBackward' , SearchPattern );
  {$ENDIF PATCH_I18N}

	sel := RPM^.SelectItem;
	repeat
		Dec( sel );
		if (sel < 1) then begin
			sel := RPM^.NumItem;
		end;
		RPM_Item := RPMLocateByPosition( RPM, sel );
		if 0 < Pos( UpCase( Key ) , UpCase( RPM_Item^.msg ) ) then begin
			RPM^.SelectItem := sel;
			SearchPattern := key;
			break;
		end;
	until ( sel = RPM^.SelectItem );

	RPMReposition_center( RPM );
	DisplayMenu(RPM);
end;

Procedure RPMSearchParentKey( RPM: RPGMenuPtr );
var
	len_currentindent: Integer;
	sel: Integer;
	RPM_Item: RPGMenuItemPtr;
	key: String;
	len_indent: Integer;
begin
	sel := RPM^.SelectItem;
	RPM_Item := RPMLocateByPosition( RPM, sel );
	if ( 0 <= RPM_Item^.indent ) then begin
		len_currentindent := RPM_Item^.indent;
	end else begin
		key := RPM_Item^.msg;
		if ( #0 = key[1] ) then begin
			key := Copy( key , 2 , Length( key ) );
		end else if ( ( ')' = key[2] ) and ( ' ' = key[3] ) ) then begin
			key := Copy( key , 3 , Length( key ) );
		end;
		len_currentindent := DeleteWhiteSpace( key );
	end;
	repeat
		Dec( sel );
		if (sel < 1) then begin
			sel := 1;
			break;
		end;
		RPM_Item := RPMLocateByPosition( RPM, sel );
		if ( 0 <= RPM_Item^.indent ) then begin
			len_indent := RPM_Item^.indent;
		end else begin
			key := RPM_Item^.msg;
			if ( #0 = key[1] ) then begin
				key := Copy( key , 2 , Length( key ) );
			end else if ( ( ')' = key[2] ) and ( ' ' = key[3] ) ) then begin
				key := Copy( key , 3 , Length( key ) );
			end;
			len_indent := DeleteWhiteSpace( key );
		end;
		if len_indent < len_currentindent then begin
			RPM^.SelectItem := sel;
			break;
		end;
	until ( sel = RPM^.SelectItem );

	RPMReposition_center( RPM );
	DisplayMenu(RPM);
end;
{$ENDIF PATCH_GH}


{$IFDEF PATCH_GH}
Function SelectMenu(RPM: RPGMenuPtr): LongInt;
{$ELSE PATCH_GH}
Function SelectMenu(RPM: RPGMenuPtr): integer;
{$ENDIF PATCH_GH}
	{This function will allow the user to browse through the menu and will}
	{return a value based upon the user's selection.}
var
	getit: char;		{Character used to store user input}
{$IFDEF PATCH_GH}
	r: LongInt;		{The value we'll be sending back.}
{$ELSE PATCH_GH}
	r: integer;		{The value we'll be sending back.}
{$ENDIF PATCH_GH}
	m: RPGMenuKeyPtr;
	UK: Boolean;		{Has a special MenuKey been pressed?}
begin
{$IFDEF PATCH_GH}
	RPMReposition_center( RPM );
{$ENDIF PATCH_GH}

	{The menu is now active!}
	RPM^.Active := True;

	{Show the menu to the user.}
	DisplayMenu(RPM);

	{Initialize UK}
	UK := False;

	{ Flush the buffer to prevent deviant keystrings. }
{$IF DEFINED(ASCII_SDL)}
	DoFlip;
{$ELSEIF DEFINED(GUIMSWINMODE)}
	while w32crt.keypressed do w32crt.readkey;
{$ELSE}
	while keypressed do readkey;
{$ENDIF}

	{Start the loop. Remain in this loop until either the player makes a selection}
	{or cancels the menu using the ESC key.}
	repeat
{$IF DEFINED(ASCII_SDL)}
		DoFlip;
{$ENDIF}

		{Read the input from the keyboard.}
		getit := RPGKey;

{$IFDEF PATCH_GH}
		{Check to see if a special MENU KEY has been pressed.}
		if RPM^.FirstKey <> Nil then begin
			m := RPM^.FirstKey;
			while m <> Nil do begin
				if getit = m^.k then begin
					UK := True;
					r := m^.value;
				end;
				m := m^.next;
			end;
		end;
		if UK then begin
			break;
		end;

		{Certain keys need processing- if so, process them.}
		if (KBD_NONE = getit) then begin end
		else if (KeyMap[ KMC_MenuUp    ].KCode = getit) then begin RPMUpKey( RPM ); end
		else if (KeyMap[ KMC_MenuDown  ].KCode = getit) then begin RPMDownKey( RPM ); end
		else if (KeyMap[ KMC_PageUp    ].KCode = getit) then begin RPMPgUpKey( RPM ); end
		else if (KeyMap[ KMC_PageDown  ].KCode = getit) then begin RPMPgDownKey( RPM ); end
		else if (KeyMap[ KMC_ScrollUp  ].KCode = getit) then begin RPMScrollUpKey( RPM ); end
		else if (KeyMap[ KMC_ScrollDown].KCode = getit) then begin RPMScrollDownKey( RPM ); end
		else if (KeyMap[ KMC_SearchParent   ].KCode = getit) then begin RPMSearchParentKey( RPM ); end
		else if (KeyMap[ KMC_SearchBackward ].KCode = getit) then begin RPMSearchBackwardKey( RPM ); end
		else if (KeyMap[ KMC_SearchForward  ].KCode = getit) then begin RPMSearchForwardKey( RPM ); end

		{If we receive an ESC, better check to make sure we're in a}
		{cancelable menu. If not, convert the ESC to an unused key.}
		else if (KBD_ESC = getit) then begin if RPM^.Mode = RPMNoCancel then getit := 'Q'; end
		{ If we get a backspace, conver that to ESC. }
		else if (KBD_BS = getit) then begin if RPM^.Mode <> RPMNoCancel then getit := KBD_ESC; end
		{ Convert enter to space. }
		else if (KBD_RET = getit) then begin getit := ' '; end
		else if (KBD_LF = getit) then begin getit := ' '; end;
{$ELSE PATCH_GH}
		{Certain keys need processing- if so, process them.}
		case getit of
			{Selection Movement Keys}
			'8': RPMUpKey(RPM);
			'2': RPMDownKey(RPM);

			{If we receive an ESC, better check to make sure we're in a}
			{cancelable menu. If not, convert the ESC to an unused key.}
			#27: If RPM^.Mode = RPMNoCancel then getit := 'Q';
		end;

		{Check to see if a special MENU KEY has been pressed.}
		if RPM^.FirstKey <> Nil then begin
			m := RPM^.FirstKey;
			while m <> Nil do begin
				if getit = m^.k then begin
					UK := True;
					r := m^.value;
				end;
				m := m^.next;
			end;
		end;
{$ENDIF PATCH_GH}

	{Check for a SPACE or ESC.}
	until (getit = ' ') or (getit = #27) or UK;

	{The menu is no longer active.}
	RPM^.Active := False;

	{We have to send back a different value depending upon whether a selection}
	{was made or the menu was cancelled. If an item was selected, return its}
	{value field. The value always returned by a cancel will be -1.}
	{If a MenuKey was pressed, r already contains the right value.}
	if getit = ' ' then begin
			r := RPMLocateByPosition(RPM,RPM^.selectitem)^.value;
		end
	else if not UK then
		r := -1;

	if RPM^.mode <> RPMNoCleanup then begin
		{Remove the menu from the display. I'm gonna use Window for this, since}
		{ClrScr in this language doesn't take paramters. Bummer.}

		SetMenuClipZone( RPM );
{$IF DEFINED(ASCII_SDL)}
		ascii_sdl.ClrScr;
{$ELSEIF DEFINED(GUIMSWINMODE)}
		w32crt.ClrScr;
{$ELSE}
		ClrScr;
{$ENDIF}

		{If there's an associated description box, clear that too.}
		if RPM^.dx1 > 0 then begin
{$IF DEFINED(ASCII_SDL)}
			ascii_sdl.Window( RPM^.DX1 , RPM^.DY1 , RPM^.DX2 , RPM^.DY2 );
			ascii_sdl.ClrScr;
{$ELSEIF DEFINED(GUIMSWINMODE)}
			w32crt.Window(RPM^.DX1,RPM^.DY1,RPM^.DX2,RPM^.DY2);
			w32crt.ClrScr;
{$ELSE}
			Window(RPM^.DX1,RPM^.DY1,RPM^.DX2,RPM^.DY2);
			ClrScr;
{$ENDIF}
		end;
	end;

	{Reset the window to normal values}
	MaxClipZone;

	SelectMenu := r;
end;

Procedure RPMSortAlpha(RPM: RPGMenuPtr);
	{Given a menu, RPM, sort its items based on the alphabetical}
	{order of their msg fields.}
	{I should mention here that I haven't written a sorting}
	{algorithm in years, and only once on a linked list (CS assignment).}
	{I think this is an insertion sort... I checked on internet for}
	{examples of sorting techniques, found a bunch of contradictory}
	{information, and decided to just write the easiest thing that}
	{would work. Since we're dealing with a relatively small number}
	{of items here, speed shouldn't be that big a concern.}
var
	sorted: RPGMenuItemPtr;	{The sorted list}
	a,b,c,d: RPGMenuItemPtr;{Counters. We always need them, you know.}
	youshouldstop: Boolean;	{Can you think of a better name?}
begin
	{Initialize A and Sorted.}
	a := RPM^.firstitem;
	Sorted := Nil;

	while a <> Nil do begin
		b := a;		{b is to be added to sorted}
		a := a^.next;	{increase A to the next item in the menu}

		{Give b's Next field a value of Nil.}
		b^.next := nil;

		{Locate the correct position in Sorted to store b}
		if Sorted = Nil then
			{This is the trivial case- Sorted is empty.}
			Sorted := b
		else if UpCase( b^.msg ) < Upcase( Sorted^.msg ) then begin
			{b should be the first element in the list.}
			c := sorted;
			sorted := b;
			sorted^.next := c;
			end
		else begin
			{c and d will be used to move through Sorted.}
			c := Sorted;

			{Locate the last item lower than b}
			youshouldstop := false;
			repeat
				d := c;
				c := c^.next;

				if c = Nil then
					youshouldstop := true
				else if UpCase( c^.msg ) > UpCase( b^.msg ) then begin
					youshouldstop := true;
				end;
			until youshouldstop;
			b^.next := c;
			d^.next := b;
		end;
	end;
	RPM^.firstitem := Sorted;
end;

{$IFDEF PATCH_CHEAT}
Procedure RPMSortAlpha_withSubItem(RPM: RPGMenuPtr);
var
	Sorted: RPGMenuItemPtr;
	NextBranches: RPGMenuItemPtr;
	CurrentBranch_Top, CurrentBranch_Bottom: RPGMenuItemPtr;
	TmpBranch_Top, TmpBranch_Bottom: RPGMenuItemPtr;
	TmpNextBranches: RPGMenuItemPtr;
	youshouldstop: Boolean;	{Can you think of a better name?}
begin
	NextBranches := RPM^.firstitem;
	Sorted := NIL;

	while NextBranches <> NIL do begin
		{  Take a current branch to 'CurrentBranch_Top' and 'CurrentBranch_Bottom', }
		{ and get the top of next branches to 'NextBranches'. }
		CurrentBranch_Top    := NextBranches;
		CurrentBranch_Bottom := CurrentBranch_Top;
		while (NIL <> CurrentBranch_Bottom^.next) and (#$0 = CurrentBranch_Bottom^.next^.msg[1]) do begin
			CurrentBranch_Bottom := CurrentBranch_Bottom^.next;
		end;
		NextBranches               := CurrentBranch_Bottom^.next;
		CurrentBranch_Bottom^.next := NIL;

		{ Locate the correct position in Sorted to store 'CurrentBranch_Top'. }
		if Sorted = NIL then begin
			Sorted := CurrentBranch_Top;
		end else if CurrentBranch_Top^.msg < Sorted^.msg then begin
			{ CurrentBranch_Top/CurrentBranch_Bottom should be the first element in the list. }
			TmpNextBranches := Sorted;
			Sorted          := CurrentBranch_Top;
			CurrentBranch_Bottom^.next := TmpNextBranches;
		end else begin
			{ Locate the last item lower than CurrentBranch_Top. }
			TmpNextBranches := Sorted;
			youshouldstop := false;
			repeat
				TmpBranch_Top    := TmpNextBranches;
				TmpBranch_Bottom := TmpBranch_Top;
				while (NIL <> TmpBranch_Bottom^.next) and (#$0 = TmpBranch_Bottom^.next^.msg[1]) do begin
					TmpBranch_Bottom := TmpBranch_Bottom^.next;
				end;
				TmpNextBranches := TmpBranch_Bottom^.next;

				if TmpNextBranches = NIL then
					youshouldstop := true
				else if CurrentBranch_Top^.msg < TmpNextBranches^.msg then begin
					youshouldstop := true;
				end;
			until youshouldstop;
			CurrentBranch_Bottom^.next := TmpNextBranches;
			TmpBranch_Bottom^.next := CurrentBranch_Top;
		end;
	end;
	RPM^.firstitem := Sorted;
end;
{$ENDIF PATCH_CHEAT}

{$IFDEF PATCH_GH}
Function SetItemByValue( RPM: RPGMenuPtr ; V: LongInt ): RPGMenuItemPtr;
{$ELSE PATCH_GH}
Function SetItemByValue( RPM: RPGMenuPtr ; V: Integer ): RPGMenuItemPtr;
{$ENDIF PATCH_GH}
	{ Search through the list, and set the SelectItem }
	{ field to the first menu item which matches V. }
var
	T: Integer;
	MI: RPGMenuItemPtr;
begin
	if RPM = Nil then exit;

	MI := RPM^.FirstItem;
	T := 1;

	while (MI <> Nil) and (MI^.Value <> V) do begin
		MI := MI^.Next;
		Inc( T );
	end;

	if MI <> Nil then begin
		RPM^.SelectItem := T;

		if (RPM^.selectitem < RPM^.topitem) or ((RPM^.selectitem - RPM^.topitem) > MenuHeight( RPM ) ) then begin
			{Determine an appropriate new value for topitem.}
			RPMReposition(RPM);
		end;
	end;

	SetItemByValue := MI;
end;

Procedure SetItemByPosition( RPM: RPGMenuPtr ; N: Integer );
	{ Search through the list, and set the SelectItem }
	{ field to the Nth menu item. }
begin
	if RPM = Nil then exit;

{$IFDEF PATCH_GH}
	if RPM^.NumItem < N then begin
		RPM^.SelectItem := RPM^.NumItem;
	end else if N < 1 then begin
		RPM^.SelectItem := 1;
	end else begin
		RPM^.SelectItem := N;
	end;
	if RPM^.NumItem < RPM^.TopItem then begin
		RPM^.TopItem := RPM^.NumItem;
	end else if RPM^.TopItem < 1 then begin
		RPM^.TopItem := 1;
	end;
	if (RPM^.SelectItem < RPM^.TopItem) or ((RPM^.SelectItem - RPM^.TopItem + 1) > MenuHeight( RPM ) ) then begin
		{Determine an appropriate new value for topitem.}
		RPMReposition(RPM);
	end;
{$ELSE PATCH_GH}
	if N <= RPM^.NumItem then begin
		RPM^.SelectItem := N;

		if (RPM^.selectitem < RPM^.topitem) or ((RPM^.selectitem - RPM^.topitem) > MenuHeight( RPM ) ) then begin
			{Determine an appropriate new value for topitem.}
			RPMReposition(RPM);
		end;
	end;
{$ENDIF PATCH_GH}
end;


{$IFDEF PATCH_GH}
Function BuildFileMenu( RPM: RPGMenuPtr; const SearchPattern: String; N: Integer ): Integer;
	{ Do a DosSearch for files matching SearchPattern, then add }
	{ each of the files found to the menu. }
var
	F: SearchRec;
{$ELSE PATCH_GH}
Procedure BuildFileMenu( RPM: RPGMenuPtr; const SearchPattern: String );
	{ Do a DosSearch for files matching SearchPattern, then add }
	{ each of the files found to the menu. }
var
	F: SearchRec;
	N: Integer;
{$ENDIF PATCH_GH}
begin
{$IFDEF PATCH_GH}
{$ELSE PATCH_GH}
	N := 1;
{$ENDIF PATCH_GH}
	FindFirst( SearchPattern , AnyFile , F );

	While DosError = 0 do begin
{$IFDEF PATCH_I18N}
  {$IFDEF PATCH_GH}
		AddRPGMenuItem( RPM , TextDecode(F.Name) , N , F.Name );
  {$ELSE PATCH_GH}
		AddRPGMenuItem( RPM , TextDecode(F.Name) , N );
  {$ENDIF PATCH_GH}
{$ELSE PATCH_I18N}
  {$IFDEF PATCH_GH}
		AddRPGMenuItem( RPM , F.Name , N , F.Name );
  {$ELSE PATCH_GH}
		AddRPGMenuItem( RPM , F.Name , N );
  {$ENDIF PATCH_GH}
{$ENDIF PATCH_I18N}
		Inc(N);
		FindNext( F );
	end;

	FindClose( F );
{$IFDEF PATCH_GH}
	BuildFileMenu := N;
{$ELSE PATCH_GH}
{$ENDIF PATCH_GH}
end;
{$IFDEF PATCH_GH}
Function BuildFileMenu( RPM: RPGMenuPtr; const SearchPattern: String ): Integer;
begin
	BuildFileMenu := BuildFileMenu( RPM , SearchPattern , 1 );
end;
{$ENDIF PATCH_GH}

Function SelectFile( RPM: RPGMenuPtr ): String;
	{ RPM is a menu created by the BuildFileMenu procedure. }
	{ So, select one of the items and return the item name, which }
	{ should be a filename. }
var
	N: Integer;	{ The number of the file selected. }
	Name: String;	{ The name of the filename selected. }
{$IFDEF PATCH_CHEAT}
	EnableKey: Boolean;
	P: Integer;
{$ENDIF PATCH_CHEAT}
begin
{$IFDEF PATCH_CHEAT}
	EnableKey := False;
	if Cheat_Restore_AddMenuKey and (Pos(' ',RPM^.FirstItem^.msg) < 1) then begin
		EnableKey := True;
		AlphaKeyMenu( RPM );
	end;
{$ENDIF PATCH_CHEAT}

	{ Do the menu selection first. }
	N := SelectMenu( RPM );

	if N = -1 then begin
		{ Selection was canceled. So, return an empty string. }
		Name := '';
	end else begin
		{ Locate the selected element of the menu. }
{$IFDEF PATCH_CHEAT}
		Name := SetItemByValue( RPM, N )^.msg;
		if EnableKey then begin
			P := Pos(' ',Name) + 1;
			Name := Copy( Name, P, Length(Name)-P+1 );
		end;
{$ELSE PATCH_CHEAT}
		Name := RPMLocateByPosition(RPM,RPM^.SelectItem)^.msg;
{$ENDIF PATCH_CHEAT}
	end;

	SelectFile := Name;
end;


initialization
begin
{$IFDEF DEBUG}
	ErrorMessage_fork('DEBUG: conmenus.pp');
{$ENDIF DEBUG}
end;

finalization
begin
{$IFDEF DEBUG}
	ErrorMessage_fork('DEBUG: conmenus.pp(finalization)');
{$ENDIF DEBUG}
end;

end.
